function [cycle, filteredy]=detrendHP(y,per,q);
% AFTER STOCK'S AND WATSON'S GAUSS PROCEDURE
%
% /* -- Procedure for producing one-sided detrended versions of a 
%       series, using a white-noise + I(2) model.  This produces a 
%       very smooth version of the trend component.  The two-sided 
%       version of the model produces the HP filter.
% 
%       A nice description of what this does is given in 
%       Harvey and Jaeger, JAE, July-Sep. 1993, pp. 231-248
% 
%       Inputs:
%       y = series to be detrended
%       q = relative variance of I(2) component
%           Note: HP quarterly uses q=.000625 (Kydland Prescott)
%                 for monthly data a value of q=.00000075
%                 matches quarterly gain at 50%, 80% and 90% periods
% */
if nargin==2; 
   switch per;
      case {.000625,12}; q=.000625;
      case {.00000075,4}; q=.00000075;
      otherwise; disp('The period should be 4 or 12, assumed quarterly (4)'); q=.000625;
   end;
end;
      
vague=1e+4; %@ Vague Prior on I(2) component @
n=length(y);
%@ -- Initialize System Matrices -- @
f=[2 -1; 1 0];
x=zeros(2,1);
p=vague*ones(2);
p(1)=vague+q;
filteredy=repmat(nan,n,1);

for i = 1:n;
	x=f*x;          %    @ x(t/t-1) @
	p=f*p*f';        %
	p(1)=p(1)+q;  %  @ p(t/t-1) @
	if isnan(y(i))== 0;
		h=p(1)+1;       %  @ variance of y @
		e=y(i)-x(1);       % @ innovation @
		k=p(:,1)/h;         %@ kalman gain @
		x=x+k*e;         %   @ x(t/t) @
		p=p-(k*p(1,:));   %  @ p(t/t) @
	end;
	filteredy(i)=x(1);
end;
cycle=y-filteredy;
