
function se=sebpass(x,xtran,tcode,n,nar,updc,lpdc);
% AFTER STOCK'S AND WATSON'S GAUSS PROCEDURE  2/13/03
% /*  Sebpass.gss
%     Compute SEs for Bandpass data (using AR padding) -- 
%     SEs associated with "endpoint" problems associated with
%     estimating pre and postsample variables
% */
% 
% @ -- Compute BP filter weights -- @

x=makeCol(x); t=length(x);

if (t < n);
    disp('Rows of X are less than n is SEpass; Cannot compute SEs');
    se=repmat(nan,t,1);
end;

avec=bpweight(updc,lpdc,n);      % @ Bandpass weights @  
%@ -- Compute 1 sided weights -- @
w=avec(n+2:end);    
xma=ar2ma(xtran,tcode,n,nar);  % @ MA Rep computed from AR -- truncated at n 
                                  %  Normalized with SD of error = 1 @
c=zeros(n);
for i=1:n; c(i:n,i)=xma(1:n+1-i); end;

se=zeros(t,1);
for i = 1:t; % @ Find Distance from end @
    j=t-i+1; j=min([j i]);
    if j <= n;
        wc=w'*c(:,j:n); v=wc*wc'; se(i)=sqrt(v);
    end;
end;

%============================================================

function xma=ar2ma(yf,tcode,n,nar);
% AFTER STOCK'S AND WATSON'S GAUSS PROCEDURE  2/13/03
% /* AR2MA -- Compute MA Rep from Estimated AR
%             -- truncate after n terms 
%             -- Allow Levels or First Differences 
%              MA Rep computed from AR -- truncated at n 
%              Normalized with SD of error = 1
%            -- Pad Data series y out using AR Forecasts and Backcasts
%      yf -- data to use in AR (levels or diffs)
%      fcode -- 1 if yf is first dif of y
%               otherwise yf=y
%      n -- number of terms for MA rep
%      nar -- order of autoregression to use
% */

%@ -- Compute MA Rep for Transformed Series -- @
xma=repmat(nan,n,1);
%@ Pad out future @
w=yf(nar+1:length(yf));
x=ones(length(w),1);
for i=1:nar; x=[x yf(nar+1-i:length(yf)-i)]; end;
beta=invpd(x'*x)*(x'*w); e=w-x*beta;
se=sqrt(e'*e/(size(x,1)-size(x,2)));
beta=beta(2:end);
comp=zeros(nar);    comp(1,1:nar)=beta'; %  @ Companion Matrix @
if nar > 1; comp(2:nar,1:nar-1)=eye(nar-1); end;
z=zeros(nar,1);
z(1)=1;
xma(1)=1;
for i=2:n; z=comp*z; xma(i)=z(1); end;
xma=xma*se;
if tcode == 1; xma=cumsum(xma); end;

