function L=gradKlmNruProd(theta,y,X,Z,unLags,lambda);
%
% log-lklhd function to compute the numerical gradient for the NAIRU, Harvey's notation
%
% measurement: y_t = z'a_t + d_t + e_t;   d_t=X_t*gamma; var(e) = H
% transition:  a_t = T*a_t-1 + c_t + u_t;   c_t=Z_t'b;     var(u) = lambda*H = Q
%

theta=makeCol(theta); nobs=length(y);
gmma=theta(1:size(X,2)); H=theta(size(X,2)+1); b=theta(size(X,2)+2:end);
u=y-X*gmma; % d_t=X*gmma

z=-theta(2:unLags+1);
T=zeros(unLags); T(1)=1; c=Z*b;
Q=zeros(unLags); Q(1)=lambda*theta(size(X,2)+1);
if unLags>1; T(2:end,1:end-1)=eye(unLags-1); end;

mu=repmat(nan,nobs,1); F=mu; atgtm1=repmat(nan,unLags,nobs); atgt=atgtm1; L=repmat(nan,nobs,1);
ptgtm1=repmat(nan,[unLags,unLags,nobs]); ptgt=ptgtm1;  
for t = 1:nobs;
    % Prediction
    if t==1; atgtm1(:,t)=repmat(0,unLags,1); ptgtm1(:,:,t)=zeros(unLags); ptgtm1(1,1,1)=1; 
    else; 
    	atgtm1(:,t)=T*atgt(:,t-1) + [c(t);zeros(unLags-1,1)]; ptgtm1(:,:,t)=T*ptgt(:,:,t-1)*T'+Q; %...+Q or +lambda*H
    end;
    % Updating
    mu(t) = u(t) - z'*atgtm1(:,t);
    F(t) = z'*ptgtm1(:,:,t)*z+H;
    atgt(:,t) = atgtm1(:,t)+ptgtm1(:,:,t)*z*pinv(F(t))*mu(t);
    ptgt(:,:,t) = ptgtm1(:,:,t)*( eye(unLags)-z*pinv(F(t))*z'*ptgtm1(:,:,t) );
    L(t) = .5*( log(F(t)) + mu(t).^2./F(t) );
end;

