function [s,omega] = spectrum(x,q,graphIndex,labs);
% [s,omega] = spectrum(x,q,graphIndex,labs);
% computes the spectrum of x using the Bartlett formula [6.3.15], Hamilton, p. 167
%
lll=[repmat('Series',size(x,2),1) int2str((1:size(x,2))')];
switch nargin;
    case 1; q = round(length(x)/4); graphIndex=0; labs=lll;
    case 2; graphIndex=0; labs=lll;
    case 3; labs=lll;
end;
gridS = 0.01; omega = (0:gridS:pi)'; weights = 1-(1:q)'/(q+1);
s = repmat(nan,length(omega),size(x,2)); 

for j=1:size(x,2);
    acvX = acv(x(:,j),q);
	for i = 1:length(omega);
        s(i,j) = ( var(x(:,j))+2*sum(weights.*acvX.*cos(omega(i)*(1:q)')) )/(2*pi);
	end;
end;

if graphIndex;
    J=omega*length(x)/(2*pi); 
    figure; h=myPlot(J,s); title('Spectrum'); axis tight; axis 'auto y'; legend(labs);
    if length(h)>2; set(h(3),'linestyle','-'); end;
    xlabel('Frequency (in periods, j = \omega_jT/(2\pi), \omega = 0 - \pi)'); print -depsc .\figs2\fSpectr.eps;
end;
%======================================================================
% function ACVFunction=acv(xSeries,nLags);
% Calculates AC for x (based on acf)
function [ACVFunction, bartlettBound]=acv(xSeries,nLags);
T=length(xSeries);
for i=1:nLags
    xCurrent=xSeries(1:T-i); xFuture=xSeries(1+i:T);
    CorrMatrix=cov(xCurrent,xFuture)*(T-i)/T;
    ACVFunction(i,1)=CorrMatrix(2,1);
end;


