
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; initSeed; tic; fprintf('\n'); diaryName=['.\out\' mfilename '.out']; % delete(diaryName); 
diary(diaryName); 
disp('==================================================================================');
disp(['Running  ' mfilename '.m                      ' datestr(now)]); format compact; tic;
disp('==================================================================================');
dataFileName='pcdata'; paramsFileName='params'; cc=customColors;
load(['.\data\' paramsFileName]); dataFile=load(['.\data\' dataFileName '.txt']); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Set Parameters 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
prodIndex=1;        % 1 non farm, 2 manufacturing
constIndex=0;       % include constant in the Zvars? (explanatory variables) 
keyIndex=0;         % insert keyboard command?
inflExpIndex=1;     % inflation expectations: 1 random walk, 2 AR forecasts, 3 AR forecasts in differences 
varEstIndex=0;
prodIndKF=1;        % productivity transform variable in the state eqn of KF: 1 diff, 2 diffdiff, 3 diff & diffdiff, 4 level, 5 level bandpass
betaDisc=.95;
unIndex=1;
prodLambda=.005;  
nruLambda=0.03; 
nruProdLambda=.01; 
shoxLags=[1 1]';  
unLags=[1 4]';
inflLags=[1 4]'; 
prodLags=[1 1]'; if prodIndKF==3; prodLags=repmat(prodLags,1,2); end;
yearEnd=[2002];
%inflIndex=3;
% frequency=1;
persForward=45;
robErrorLags=0;  % set lags for NW std errors (0=white fomula, negative=OLS)
maxHfore=12;    % max periods ahead to forecast
splineSize=knots+splineOrder+1; % SSW splines: 3 knots, splineOrder = 3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Time Period:  %4.0f  -- %4.0f\n',yearStart,yearEnd);
fprintf('Productivity Trend Lambda:  %1.3f   NAIRU RW Lambda:  %1.3f   Productivity NAIRU Lambda:  %1.3f\n',prodLambda,nruLambda,nruProdLambda);
if splineIndex~=1; knots=0; splineOrder=0; splineSize=1; end;
if shoxIndex==1; maxLags=max([inflLags(2),unLags(2),shoxLags(2)]); ... % DO I NEED MAXLAGS AT ALL?
   else; maxLags=max([inflLags(2),unLags(2)]); end;
maxUn=max(unLags); maxInfl=max(inflLags);
allLags=[unLags, inflLags];
freqOps;
transformDataNru2; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Phillips Curve Regressions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
inpSpl=struct('X',X,'y',y,'unLags',unLags,'inLags',inflLags,'nLags',allLags,'shoxIndex',shoxIndex,...
    'splineIndex',splineIndex,'knots',knots,'splineOrder',splineOrder,'rErrors',robErrorLags);  

[betaOLS,ssrUnrestricted,degreesOfFrdm,vcvOLS,res,seOLS,ftstat,ftname]=nairuRegSpli3(inpSpl);
deltaSE= deltaM2(betaOLS,vcvOLS,unLags,X(:,1:splineSize),splineIndex); norm95=norminv(.975,0,1);
fStats;
splineTrendFit=X(:,1:splineSize)*betaOLS(1:splineSize); 
unSum= sum( betaOLS(splineSize+1:splineSize+maxUn) );
nairu= splineTrendFit/(-unSum);

unSumVec=[zeros(splineSize,1);ones(maxUn,1); zeros(length(betaOLS)-maxUn-splineSize,1)];
unSumSE=sqrt(unSumVec'*vcvOLS*unSumVec); unSumTstat=unSum/unSumSE;
fprintf('sum of unemployment:   %5.5g    t stat: %5.5g   P value:  %5.5g\n',unSum, unSumTstat,2*(1-normcdf(abs(unSumTstat))) );
fprintf('s.e. on sum of unempl.:       %5.5g   se on constant:     %5.5g\n',unSumSE, sqrt(vcvOLS(1))); 
ci1=betaOLS-1.96*sqrt(diag(vcvOLS)); ci2=betaOLS+1.96*sqrt(diag(vcvOLS)); 
[impRespOLSSlo,impSlo]=irs(betaOLS,allLags,persForward,2); impRespOLSSlo=cumsum(impRespOLSSlo);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%  Exact Anderson-Rubin method & Delta
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if splineIndex==1; timeFrame=(1:frequency:length(y))'; nairuCId=[nairu,nairu]+norm95*kron([-1, 1],deltaSE);
   fprintf('NAIRU is Time-Varying Spline\n');
else; timeFrame=1; ons=ones(size(X,1),1); nairuCId=[nairu,nairu]+norm95*kron([-1, 1],deltaSE*ons); end;
if arIndex==1;
	inputSearch = struct('nairuMin',0,'nairuMax',10,'nairuMesh',.01,'tolerance',1e-4,'X',X,'y',y,'allLags',allLags,...
       'searchMethod',sMethod,'ubar',nairu,'seUbar',deltaSE,'splineIndex',splineIndex,'splineSize',splineSize,'timeFrame',timeFrame);
	[arCI,n,arStat]=ciSearch5(inputSearch);
	%% interpolation of CI's
	if splineIndex==1;
		xCoordNow=time(timeFrame); yCoordNow=arCI; xCoordDesired=time;
		nairuCIar=interp1(xCoordNow,yCoordNow,xCoordDesired,'spline');
	else; nairuCIar=kron(arCI,ons); end;
else nairuCIar=zeros(size(fiellerCI));
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Productivity Phillips Curve Regressions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
inpProd=struct('X',Xprod,'y',y,'nLags',[prodLags allLags],'shoxIndex',shoxIndex,'splineIndex',splineIndex,'knots',knots,...
    'splineOrder',splineOrder,'rErrors',robErrorLags);  

[betaOLSp,ssrUnrestrictedp,degreesOfFrdmp,vcvp,resp,seOLSp,ftstatp,ftnamep]=nairuRegProd(inpProd);
nruP=-(betaOLSp(1)+Xprod(:,2:max(prodLags)+1)*betaOLSp(2:max(prodLags+1)))/sum(betaOLSp(max(prodLags)+2:max(prodLags)+1+max(unLags)));
figure; myPlot(time,[nruP Xprod(:,prodLags(2)+2) unTrend]); title('Productivity NAIRU (Regression)'); legend('NAIRU','Unemployment','Unemployment Trend'); axis([-inf inf 0 11]);

prodSum= sum( betaOLSp(splineSize+1:splineSize+prodLags(2)) );
prodSumVec=[zeros(splineSize,1);ones(prodLags(2),1); zeros(length(betaOLSp)-prodLags(2)-splineSize,1)];
prodSumSE=sqrt(prodSumVec'*vcvp*prodSumVec); prodSumTstat=prodSum/prodSumSE;
fprintf('sum of productivity:   %5.5g    t stat: %5.5g   P value:  %5.5g\n',prodSum, prodSumTstat,2*(1-normcdf(abs(prodSumTstat))) );
fprintf('s.e. on sum of prod.:       %5.5g   se on constant:     %5.5g\n',prodSumSE, sqrt(vcvp(1))); 
unSumVecP=[zeros(splineSize+diff(prodLags)+1,1);ones(maxUn,1); zeros(length(betaOLSp)-maxUn-splineSize-diff(prodLags)-1,1)]; 
unSumP=sum(betaOLSp.*unSumVecP); unSumSEP=sqrt(unSumVecP'*vcvp*unSumVecP); unSumTstatP=unSumP/unSumSEP;;
fprintf('sum of unemployment:   %5.5g    t stat: %5.5g   P value:  %5.5g\n',unSumP, unSumTstatP,2*(1-normcdf(abs(unSumTstatP))) );
fprintf('s.e. on sum of unempl.:       %5.5g   se on constant:     %5.5g\n',unSumSEP, sqrt(vcvp(1))); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%  Exact Anderson-Rubin method & Delta
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
timeFrameP=(1:frequency:length(y))'; fprintf('NAIRU is Time-Varying Spline\n');
if arIndex==1&prodIndKF==4;
	inputSearch = struct('nairuMin',0,'nairuMax',10,'nairuMesh',.01,'tolerance',1e-4,'X',Xprod,'y',y,'allLags',allLags,...
       'searchMethod',sMethod,'ubar',nairu,'seUbar',repmat(deltaSE,size(y)),'splineIndex',1,'splineSize',2,'timeFrame',timeFrameP);
	[arCIP,n,arStat]=ciSearch5(inputSearch);
	%% interpolation of CI's
	xCoordNow=time(timeFrameP); yCoordNow=arCIP; xCoordDesired=time;
	nairuCIarP=interp1(xCoordNow,yCoordNow,xCoordDesired,'spline');
else nairuCIarP=zeros(length(y),2);
end;

pVal=[0.05; 0.5; 095]; 
[lda,ldaR,tau,tauR]=mubNru(y,X,pVal);
[ldap,ldaRp,taup,tauRp]=mubNru(y,Xprod,pVal);
disp('Median Unbiased Estimates of Variance and 90% CI:'); allTau=[tau;tauR;taup;tauRp];
fprintf('RW:          %2.4f       %2.4f    %2.4f\n',tau(1),tau(2),tau(3));
fprintf('RW Robust:   %2.4f       %2.4f    %2.4f\n',tauR(1),tauR(2),tauR(3));
fprintf('Prod:        %2.4f       %2.4f    %2.4f\n',taup(1),taup(2),taup(3));
fprintf('Prod Robust: %2.4f       %2.4f    %2.4f\n',tauRp(1),tauRp(2),tauRp(3));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Kalman Random Walk
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
kalmanDataIn=[ones(length(brRegs),1) brRegs];
initGuess=kalmanDataIn\y; varGuess=var(y-kalmanDataIn*initGuess); exitflag=0; 
gradHandleI=@gradKlmNru;
if varEstIndex; mleEstimates=[initGuess; varGuess; nruLambda*varGuess]; kalmanHandle=@klmNruEst;
else; mleEstimates=[initGuess; varGuess]; kalmanHandle=@klmNru; end;
smoothHandleI=@getKlmNru; options=optimset('MaxFunEvals',300*length(mleEstimates));
for i=1:5; 
    if exitflag==0;
        if varEstIndex; [mleEstimates(:,i+1),fval1,exitflag,output] = fminsearch(kalmanHandle,mleEstimates(:,i),options,y,kalmanDataIn,maxUn);
        else; [mleEstimates(:,i+1),fval1,exitflag,output] = fminsearch(kalmanHandle,mleEstimates(:,i),options,y,kalmanDataIn,maxUn,nruLambda); end;
    end;
end;

initEstimates=mleEstimates(:,end); fprintf('Initial Estimates Generated. Lambda:  %3.3f,     MLE Estimates:\n',nruLambda); disp(initEstimates);
if varEstIndex; smPars0=[]; else; smPars0=nruLambda*initEstimates(end); end; smPars=[initEstimates; smPars0];
[AFrw,PFrw,ASrw,PSrw,muRw,fRw]=feval(smoothHandleI,smPars,y,kalmanDataIn,maxUn);
% figure; h=myPlot(time,[ASrw ASp(maxLags+1:end) graphUn]); legend('NAIRU','Productivity','Unemployment'); title(['Unemployment and the NAIRU, \lambda=' num2str(nruLambda)]);
% set(h(2),'linestyle','-'); set(h(3),'linestyle','--');  axis([-inf inf 0 11]);
bic1=(-2*fval1+(length(initEstimates)+1)*log(length(kalmanDataIn)))/length(kalmanDataIn);
fprintf('Value of log-likelihood:    %4.3f      BIC    %2.3f     var of 1step ahead pred errors    %3.3f\n',fval1,bic1,var(muRw));

%  Standard Errors: Estimates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf0=feval(gradHandleI,initEstimates,y,kalmanDataIn,maxUn,nruLambda); d2=1e-6; ks0=ASrw;
delta=d2*initEstimates; nPars=length(initEstimates); dI=eye(nPars);
for i=1:nPars;
	x1=initEstimates+delta(i)*dI(:,i);
	lf1(:,i)=feval(gradHandleI,x1,y,kalmanDataIn,maxUn,nruLambda);
   [junk,junk,AS1,junk]=feval(smoothHandleI,[x1; smPars0],y,kalmanDataIn,maxUn); ks1(:,i)=AS1;
end;
grad=-(lf1-repmat(lf0,1,length(initEstimates)))./repmat(delta',length(lf1),1);
vcv=pinv(grad'*grad);
vcvSErw=sqrt(diag(vcv));
fprintf('Delta   %4.4g \n ',d2);
disp('MLE Estimates   Std Error  P value');
disp([initEstimates vcvSErw 2*(1-normcdf(abs(initEstimates./vcvSErw)))]);

B1=sum(initEstimates(2:maxUn+1)); seB1=sqrt(sum(sum(vcv(2:maxUn+1,2:maxUn+1))));
fprintf('Sum of Coeffs on Unemployment:   %3.3f   Std Error:   %3.3f     P value:  %1.5f\n',B1,seB1,2*(1-normcdf(abs(B1/seB1))));

uZeroStatRW=maxUn*initEstimates(2:maxUn+1)'*inv(vcv(2:maxUn+1,2:maxUn+1))*initEstimates(2:maxUn+1);
fprintf('Chi2 Sum of Coeffs on Unemployment:   %3.3f   P value:  %1.5f\n',uZeroStatRW,1-chi2cdf(uZeroStatRW,maxUn));
iZeroStatRW=maxInfl*initEstimates(maxUn+2:maxUn+maxInfl+1)'*inv(vcv(maxUn+2:maxUn+maxInfl+1,maxUn+2:maxUn+maxInfl+1))*initEstimates(maxUn+2:maxUn+maxInfl+1);
fprintf('Chi2 Sum of Coeffs on Inflation:   %3.3f   P value:  %1.5f\n',iZeroStatRW,1-chi2cdf(iZeroStatRW,maxInfl));

% Delta Method
gradNRU=(ks1-repmat(ks0,1,nPars))./repmat(delta',length(ASrw),1);
varDNRU=diag(gradNRU*vcv*gradNRU'); % Ansley Kohn (1986) eqn 10 correction
varkNRU=PSrw+varDNRU;
ciAS2rw=[ASrw-1.96*sqrt(varkNRU), ASrw+1.96*sqrt(varkNRU)]; 
ciASrw=[ASrw-1.96*sqrt(PSrw), ASrw+1.96*sqrt(PSrw)]; 
kalmanGraph0;

crr2=corrcoef([ASrw ASp(maxLags+1:end)]); fprintf('Correlation between Kalman Filter NAIRU and Productivity:  %1.3f\n',crr2(2));
t=toc; mints=floor(toc/60); secs=toc-60*mints; fprintf('Split Time: %4.0f minutes, %4.2f seconds\n',mints,secs); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Kalman Random Walk Productivity
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
kalmanDataInP=[Xprod(:,1) Xprod(:,3:end) Xprod(:,2)];
initGuessP=kalmanDataInP\y; varGuessP=var(y-kalmanDataInP*initGuessP); exitflag=0; 
mleEstimatesP=[initGuessP; varGuessP]; 
options=optimset('MaxFunEvals',300*length(mleEstimates));
for i=1:5; 
    if exitflag==0;
        [mleEstimatesP(:,i+1),fval1,exitflag,output] = fminsearch(kalmanHandle,mleEstimatesP(:,i),options,y,kalmanDataInP,maxUn,nruProdLambda); end;
    end;
end;

getHandleR=@getKlmNruRegP;
mleEstimatesPfin=mleEstimatesP(:,end); smParsP=[mleEstimatesPfin; nruProdLambda*mleEstimatesPfin(end)];
[AFP,PFP,ASP,PSP,muP,fP]=feval(getHandleR,smParsP,y,kalmanDataInP,maxUn);
figure; myPlot(time,[ASP ASrw graphUn nruP]); title('Unemployment and the NAIRU'); legend('Prod','RW','Unemployment','Reg'); grid on;

%  Standard Errors: Estimates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf0P=feval(gradHandleI,mleEstimatesPfin,y,kalmanDataInP,maxUn,nruProdLambda); ks0P=ASP;
delta=d2*mleEstimatesPfin; nParsP=length(mleEstimatesPfin); dI=eye(nParsP);
for i=1:nParsP;
	x1=mleEstimatesPfin+delta(i)*dI(:,i);
	lf1P(:,i)=feval(gradHandleI,x1,y,kalmanDataInP,maxUn,nruProdLambda);
   [junk,junk,AS1P,junk]=feval(getHandleR,[x1; nruProdLambda*mleEstimatesPfin(end)],y,kalmanDataInP,maxUn); ks1P(:,i)=AS1P;
end;
gradP=-(lf1P-repmat(lf0P,1,length(mleEstimatesPfin)))./repmat(delta',length(lf1P),1);
vcvP=pinv(gradP'*gradP);
vcvSEP=sqrt(diag(vcvP));
fprintf('Delta   %4.4g \n ',d2);
disp('MLE Estimates   Std Error  P value'); pValP=2*(1-normcdf(abs(mleEstimatesPfin./vcvSEP)));
disp([mleEstimatesPfin vcvSEP pValP]);

uZeroStatP=maxUn*mleEstimatesPfin(2:maxUn+1)'*inv(vcvP(2:maxUn+1,2:maxUn+1))*mleEstimatesPfin(2:maxUn+1);
fprintf('Chi2 Sum of Coeffs on Unemployment:   %3.3f   P value:  %1.5f\n',uZeroStatP,1-chi2cdf(uZeroStatP,maxUn));
iZeroStatP=maxInfl*mleEstimatesPfin(maxUn+2:maxUn+maxInfl+1)'*inv(vcvP(maxUn+2:maxUn+maxInfl+1,maxUn+2:maxUn+maxInfl+1))*mleEstimatesPfin(maxUn+2:maxUn+maxInfl+1);
fprintf('Chi2 Sum of Coeffs on Inflation:   %3.3f   P value:  %1.5f\n',iZeroStatP,1-chi2cdf(iZeroStatP,maxInfl));

B1P=sum(mleEstimatesPfin(2:maxUn+1)); seB1P=sqrt(sum(sum(vcvP(2:maxUn+1,2:maxUn+1))));
fprintf('Sum of Coeffs on Unemployment:   %3.3f   Std Error:   %3.3f     P value:  %1.5f\n',B1P,seB1P,2*(1-normcdf(abs(B1P/seB1P))));

% Delta Method
gradNRUP=(ks1P-repmat(ks0P,1,nParsP))./repmat(delta',length(ASP),1);
varDNRUP=diag(gradNRUP*vcvP*gradNRUP'); % Ansley Kohn (1986) eqn 10 correction
varkNRUP=PSP+varDNRUP;
ciAS2P=[ASP-1.96*sqrt(varkNRUP), ASP+1.96*sqrt(varkNRUP)]; 
ciASP=[ASP-1.96*sqrt(PSP), ASP+1.96*sqrt(PSP)]; 
diffKP=diff(ciAS2P')';
kalmanGraphP;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Kalman Productivity
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% statePer=0;
kalmanData=[ones(length(brRegs),1) brRegs]; %kalmanData=[ones(length(Xprod2),1) Xprod2];
prodKHandle=@klmNruProd; smoothHandle=@getKlmNruProd; gradHandle=@gradKlmNruProd;
switch prodIndKF; case 1; initProd=-2; case 2; initProd=-2; case 3; initProd=[-2; -2]; otherwise; initProd=-2;  end;
if constIndex; Zvars=[ones(length(prodMat),1) prodMat]; mleEstimates=[initEstimates' 0 initProd' zeros(1,prodLags(2)-1)]'; %initial guesses from the estimation without productivity
else Zvars=[prodMat]; mleEstimates=[initEstimates' initProd' zeros(1,prodLags(2)-1)]'; end;
Zvars2=Zvars(maxLags-prodLags(2)+1:end,:); exitflag=0; options=optimset('MaxFunEvals',300*length(mleEstimates)); 
if keyIndex; keyboard; end;
% keyboard
for i=1:5; 
    if exitflag==0;
    [mleEstimates(:,i+1),fval2,exitflag,output] = fminsearch(prodKHandle,mleEstimates(:,i),options,y,kalmanData,Zvars2,maxUn,nruProdLambda);
    end;
end;
mleEstimates2=mleEstimates(:,end);

%  Standard Errors: Estimates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
if constIndex; smPars=nruProdLambda*mleEstimates(end-prodLags(2)-1); else smPars=nruProdLambda*mleEstimates(end-prodLags(2)); end;
[AF,PF,AS,PS,mu,f]=feval(smoothHandle,[mleEstimates2;smPars],y,kalmanData,Zvars2,maxUn); ks0=AS;

fComp=figure; h=myPlot(time,[AS ASrw graphUn]); legend('Productivity NAIRU ','Random Walk','Unemployment'); title(['Productivity-driven and Random Walk NAIRUs']); 
set(h(2),'linestyle','-','color',cc.liteGB); set(h(3),'color',cc.darkBlue,'linestyle','--'); grid on;
xlabel(['Productivity \lambda = ' num2str(nruProdLambda) ',  Random Walk \lambda = ' num2str(nruLambda)]); axis([-inf inf 0 11]);
plotData=[AS graphUn graphInflA zeros(size(graphUn))];
figure; h=myPlot(time,plotData); legend('Productivity NAIRU ','Unemployment','Inflation Trend'); title(['Productivity-driven and NAIRU and Bandpass']); 
set(h(3),'linestyle','--','color',cc.liteGB); set(h(2),'color',cc.darkBlue); 
set(h(4),'linewidth',1,'color','k'); xlabel(['Productivity \lambda = ' num2str(nruProdLambda)]); grid on;
% figure; h=myPlot(time,[mu muRw]); title('Predictive Errors'); legend('Productivity','Random Walk')

lf0=feval(gradHandle,mleEstimates2,y,kalmanData,Zvars2,maxUn,nruProdLambda); d2=1e-6;
delta=d2*mleEstimates2; nPars=length(mleEstimates2); dI=eye(nPars);
for i=1:nPars;
	x1=mleEstimates2+delta(i)*dI(:,i);
	lf1(:,i)=feval(gradHandle,x1,y,kalmanData,Zvars2,maxUn,nruProdLambda);
   [junk,junk,AS1,junk]=feval(smoothHandle,[x1;smPars],y,kalmanData,Zvars2,maxUn); ks1(:,i)=AS1;
end;
grad=-(lf1-repmat(lf0,1,length(mleEstimates2)))./repmat(delta',length(lf1),1);
vcvKF=pinv(grad'*grad);
vcvSE=sqrt(diag(vcvKF));

fprintf(['Delta   %4.4g   Lambda   %4.4f    Productivity: ' pDiffText  ' \n '],d2,nruProdLambda);
disp('MLE Estimates   Std Error  P value');
disp([mleEstimates2 vcvSE 2*(1-normcdf(abs(mleEstimates2./vcvSE)))]);
mpLags=prodLags(2)-1; pZerosStat=(mpLags+1)*mleEstimates2(end-mpLags:end)'*inv(vcvKF(end-mpLags:end,end-mpLags:end))*mleEstimates2(end-mpLags:end);
fprintf('Chi2 Stat on Each Productivity = 0 (jointly):     %4.4f    P value  %4.4f\n',pZerosStat,1-chi2cdf(pZerosStat,mpLags+1)); % 
pSumStat=sum(mleEstimates2(end-mpLags:end)).^2/sum(sum(vcvKF(end-mpLags:end,end-mpLags:end))); % 1 dg of freedom
fprintf('Chi2 Stat on Sum of Productivity = 0:     %4.4f    P value  %4.4f\n',pSumStat,1-chi2cdf(pSumStat,1)); % 
B1p=sum(mleEstimates2(2:maxUn+1)); seB1p=sqrt(sum(sum(vcvKF(2:maxUn+1,2:maxUn+1))));
fprintf('Sum of Coeffs on Unemployment:   %3.3f   Std Error:   %3.3f     P value:  %1.5f\n',B1p,seB1p,2*(1-normcdf(abs(B1p/seB1p))));
bic2=(-2*fval2+(length(mleEstimates2)+1)*log(length(kalmanData)))/length(kalmanData);

uZeroStat=maxUn*mleEstimates2(2:maxUn+1)'*inv(vcvKF(2:maxUn+1,2:maxUn+1))*mleEstimates2(2:maxUn+1);
fprintf('Chi2 Sum of Coeffs on Unemployment:   %3.3f   P value:  %1.5f\n',uZeroStat,1-chi2cdf(uZeroStat,maxUn));
iZeroStat=maxInfl*mleEstimates2(maxUn+2:maxUn+maxInfl+1)'*inv(vcvKF(maxUn+2:maxUn+maxInfl+1,maxUn+2:maxUn+maxInfl+1))*mleEstimates2(maxUn+2:maxUn+maxInfl+1);
fprintf('Chi2 Sum of Coeffs on Inflation:   %3.3f   P value:  %1.5f\n',iZeroStat,1-chi2cdf(iZeroStat,maxInfl));

fprintf('Value of log-likelihood:    %4.3f      BIC    %2.3f    var of 1step ahead pred errors:  %3.3f\n',fval2,bic2,var(mu));
fprintf(' kinda ''R^2'', productivity model: %1.3f     random walk model:  %1.3f\n',smPars/var(AS),nruLambda*initEstimates(end)/var(ASrw));

% Delta Method
gradNRU=(ks1-repmat(ks0,1,nPars))./repmat(delta',length(AS),1);
varDNRU=diag(gradNRU*vcvKF*gradNRU'); % Ansley Kohn (1986) eqn 10 correction
varkNRU=PS+varDNRU;
ciAS2=[AS-1.96*sqrt(varkNRU), AS+1.96*sqrt(varkNRU)]; 
ciAS=[AS-1.96*sqrt(PS), AS+1.96*sqrt(PS)]; 
kalmanGraph; kalmanGraph2;

diff1=diff(ciAS2rw')'; diff2=diff(ciAS2')'; 
fprintf('Mean Width RW:  %1.3f   Mean Width Prod:  %1.3f  Mean Diff in width (Prod/RW):  %1.3f\n',mean(diff1),mean(diff2),mean(diff2)/mean(diff1));
figure; h=myPlot(time,[AS ASrw graphUn -diff1 -diff2 zeros(size(AS))]); 
legend('Productivity NAIRU ','Random Walk','Unemployment','Width of RW','Width of Prod',0); title(['Productivity-driven and Random Walk NAIRUs']); 
set(h(3),'linestyle','--','color',cc.liteGB); set(h(2),'color',cc.darkBlue); grid on; set(h(end),'color','k','linewidth',1,'linestyle','-');
xlabel(['Productivity \lambda = ' num2str(nruProdLambda) ',  Random Walk \lambda = ' num2str(nruLambda)]); % axis([-inf inf 0 11]);

% Spectrum
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lgd=[repmat('Random Walk, \lambda = ',size(nruLambda)) num2str(nruLambda)];
labs=strvcat('Productivity',lgd); [specNru,omega] = spectrum([AS ASrw],min([50,round(length(AS)/2)]),1,labs); 

% Impulse Responses, Sacrifice Ratios
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[impRespRWF,impF]=irs(initEstimates,allLags,persForward,1); impRespRWF=cumsum(impRespRWF);
[impRespRWSlo,impSlo]=irs(initEstimates,allLags,persForward,2); impRespRWSlo=cumsum(impRespRWSlo);
[impRespProdF,impF]=irs(mleEstimates2,allLags,persForward,1); impRespProdF=cumsum(impRespProdF);
[impRespProdSlo,impSlo]=irs(mleEstimates2,allLags,persForward,2); impRespProdSlo=cumsum(impRespProdSlo);
betaOLSp2=[betaOLSp([1 (3:length(betaOLSp))]); betaOLSp(2)];
[impRespProdRegSlo,impSlo]=irs(betaOLSp2,allLags,persForward,2); impRespProdRegSlo=cumsum(impRespProdRegSlo);
[impRespProdRwSlo,impSlo]=irs(mleEstimatesPfin,allLags,persForward,2); impRespProdRwSlo=cumsum(impRespProdRwSlo);

sacrRatAlt=-1./(frequency*[impRespProdRegSlo(end); impRespProdRwSlo(end)]);

fprintf('Long-run Effects on Inflation of 1%% rise in unemployment for 1 year (in %%):\n');
fprintf('RW:  %5.3g       Productivity:  %5.3f\n',frequency*impRespRWF(end),frequency*impRespProdF(end));
sacrRat=-1./(frequency*[impRespRWF(end); impRespProdF(end)]);
fprintf('Implied sacrifice ratio (assuming Okun''s coeff 2):\n'); 
fprintf('RW:  %5.3g       Productivity:  %5.3f\n',sacrRat(1),sacrRat(2));
impRespFigs; resTab; resTabAlt;

% Forecasting Stuff
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allNrus=[nairu nruP ASrw ASP AS]; % collect all NRU estimates
allNrus=[allNrus(1,:); allNrus];
nruL=mklagsmine(allNrus,repmat([1;maxLags],1,size(allNrus,2))); mseSall=repmat(nan,maxHfore,2); mseSallIn=mseSall;

for hFore=1:maxHfore; 
	% Forecasting data
    nruRegs=nruL.xl;
	xFore=[un(2:end),infl(2:end)-infl(1:end-1)]; xFore=xFore(1:end-hFore+1,:); yFore=infl(1+hFore:end)-infl(1:end-hFore); yFore=yFore(maxLags+1:end);
	XF=mklagsmine(xFore,[unLags inflLags]); XFore=XF.xl; yForeLev=infl(1:end-hFore); yForeLev=yForeLev(maxLags+1:end);
	if maxLags>hFore; yFore=yFore(maxLags-hFore+1:end); XFore=XFore(maxLags-hFore+1:end,:); 
	elseif maxLags<hFore; nruRegs=nruRegs(hFore-maxLags+1:end,:); end;
        
	XForeProd=XFore; XForeProd(:,1:unLags(2))=XForeProd(:,1:unLags(2))-nruRegs(:,4*maxLags+1:end);
	XForeConst=XFore; XForeConst(:,1:unLags(2))=XForeConst(:,1:unLags(2))-nruRegs(:,1:maxLags);
	XForeRW=XFore; XForeRW(:,1:unLags(2))=XForeRW(:,1:unLags(2))-nruRegs(:,2*maxLags+1:3*maxLags);
	
	[fConst,actInfl,fittedConst]=foreTVnru(yFore,XForeConst); 
	[fProd,actInfl,fittedProd]=foreTVnru(yFore,XForeProd); 
	[fRW,actInfl,fittedRW]=foreTVnru(yFore,XForeRW); 
	fErrors2=([[fConst fProd fRW]-repmat(actInfl,1,3)]).^2; mseS=mean(fErrors2); mseS=mseS(2:end)/mseS(1); mseSall(hFore,:)=mseS;
	fErrors2In=([[fittedConst fittedProd fittedRW]-repmat(actInfl,1,3)]).^2; mseSIn=mean(fErrors2In); mseSIn=mseSIn(2:end)/mseSIn(1); mseSallIn(hFore,:)=mseSIn;
    fprintf('Rel MSEs (wrt constant), Fore Hor  %2.0f Periods:    %2.4f    %2.4f    In Sample:  %2.4f    %2.4f\n',hFore,mseS(1),mseS(2),mseSIn(1),mseSIn(2));
    % fTime=time(length(time)-length(fRW)+1:end); 
	% figure; myPlot(fTime,fErrors2); title('Forecasting Errors'); legend('Constant','Productivity','Random Walk');
	% figure; myPlot(fTime,[fConst fProd fRW actInfl]); title('\Delta \pi Forecasts'); legend('Constant','Productivity','Random Walk','Actual');
	% figure; myPlot(fTime,[fittedConst fittedProd fittedRW actInfl]); title('\Delta\pi In Sample Fitted Values');
	% legend('Constant','Productivity','Random Walk','Actual');
	%actInfl=yForeLev+actInfl; fProd=yForeLev+actInfl;
end;
relMSE=[mseSall mseSallIn];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
printFigs;
t=toc; mints=floor(toc/60); secs=toc-60*mints;
fprintf('Time: %4.0f minutes, %4.2f seconds\n',mints,secs); diary off;
