
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; initSeed; tic; fprintf('\n'); 
diaryName=['.\out\' mfilename '.out']; %delete(diaryName); 
diary(diaryName);
disp('==================================================================================');
disp(['Running  ' mfilename '.m                      ' datestr(now)]); format compact; tic;
disp('==================================================================================');
dataFileName='pcdata'; paramsFileName='params'; cc=customColors;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load(['.\data\' paramsFileName]); dataFile=load(['.\data\' dataFileName '.txt']); 
prodIndex=1;    % 1 non farm, 2 manufacturing
constIndex=0;   % include constant in the Zvars? (explanatory variables) 
prodDiffIndex=1; % 1 difference, 2 ratio, 3 productivity trend
keyIndex=0;     % insert keyboard command?
inflExpIndex=1;  % inflation expectations: 1 random walk, 2 AR forecasts 
prodIndKF=4; % productivity transform variable in the state eqn of KF: 1 aspirations, 2 prod trend, 3 diff aspirations, 4 diff prod trend, 5 KF & KS
prodLambda=.003; % annual .025, .03  quarterly .003
nruLambda=[0.01 0.03 0.2]';  %  annual 0.3  quarterly 0.05
nruProdLambda=[0.001 .01 0.07]'; % annual  0.05 , 0.025 quarterly 0.01
shoxLags=[1 1]';  
unLags=[1 4]';
inflLags=[1 4]'; 
prodLags=[1 1]'; 
%frequency=1;
robErrorLags=0;  % set lags for NW std errors (0=white fomula, negative=OLS)
splineSize=knots+splineOrder+1; % SSW splines: 3 knots, splineOrder = 3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if splineIndex~=1; knots=0; splineOrder=0; splineSize=1; end;
if shoxIndex==1; maxLags=max([inflLags(2),unLags(2),shoxLags(2)]); ... % DO I NEED MAXLAGS AT ALL?
   else; maxLags=max([inflLags(2),unLags(2)]); end;
maxUn=max(unLags); maxInfl=max(inflLags);
allLags=[unLags, inflLags];
freqOps;
transformDataNru;
keyboard
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Kalman Productivity
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Pick initial values (No Productivity)
kalmanDataIn=[ones(length(brRegs),1) brRegs];
initGuess=kalmanDataIn\y; varGuess=var(y-kalmanDataIn*initGuess);
gradHandleI=@gradKlmNru; mleEstimates=[initGuess; varGuess]; kalmanHandle=@klmNru;
smoothHandleI=@getKlmNru; options=optimset('MaxFunEvals',300*length(mleEstimates));
ASrw=repmat(nan,length(kalmanDataIn),length(nruLambda)); 
for j=1:length(nruLambda);
   exitflag=0; 
	for i=1:5; 
        if exitflag==0;
            [mleEstimates(:,i+1),fval1,exitflag,output] = fminsearch(kalmanHandle,mleEstimates(:,i),options,y,kalmanDataIn,maxUn,nruLambda(j));
        end;
	end;
	mleEstimates1(:,j)=mleEstimates(:,end);
	initEstimates=mleEstimates(:,end); fprintf('Random Walk Lambda:  %3.3f,     MLE Estimates:\n',nruLambda(j)); disp(initEstimates);
	smPars0=nruLambda(j)*initEstimates(end); smPars=[initEstimates; smPars0];
	[AFrw,PFrw,ASrw(:,j),PSrw,muRw,fRw]=feval(smoothHandleI,smPars,y,kalmanDataIn,maxUn);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
crr2=corrcoef([ASrw ASp(maxLags+1:end)]); fprintf('Correlation between Kalman Filter NAIRU and Productivity:  %1.3f\n',crr2(2));
t=toc; mints=floor(toc/60); secs=toc-60*mints; fprintf('Split Time: %4.0f minutes, %4.2f seconds\n',mints,secs); 
%==========================================================================

kalmanData=[ones(length(brRegs),1) brRegs]; %kalmanData=[ones(length(Xprod2),1) Xprod2];
initGuess=kalmanData\y; varGuess=var(y-kalmanData*initGuess); initEstimates=[initGuess; varGuess];
prodKHandle=@klmNruProd; smoothHandle=@getKlmNruProd; gradHandle=@gradKlmNruProd;
if constIndex; Zvars=[ones(length(prodMat),1) prodMat]; smPars=[0 -2 zeros(1,prodLags(2)-1)]'; %initial guesses from the estimation without productivity
else Zvars=[prodMat]; smPars=[-2 zeros(1,prodLags(2)-1)]'; end;
Zvars2=Zvars(maxLags-prodLags(2)+1:end,:); exitflag=0; options=optimset('MaxFunEvals',300*length(initEstimates)); 
if keyIndex; keyboard; end;

AS=repmat(nan,length(kalmanData),length(nruProdLambda));
for j=1:length(nruProdLambda);
   exitflag=0;  mleEstimates=[initEstimates; smPars];
	for i=1:5; 
        if exitflag==0;
        [mleEstimates(:,i+1),fval2,exitflag,output] = fminsearch(prodKHandle,mleEstimates(:,i),options,y,kalmanData,Zvars2,maxUn,nruProdLambda(j));
        end;
	end;
	mleEstimates2(:,j)=mleEstimates(:,end); mleEstimates
    sum(mleEstimates2(2:unLags(2)+1,j))
	fprintf('Productivity Lambda:  %3.3f,     MLE Estimates:\n',nruProdLambda(j)); disp(mleEstimates2(:,end));
	smPars0=nruProdLambda(j)*mleEstimates2(end-1-constIndex,j); smPars0=[mleEstimates(:,end); smPars0];
	[AF,PF,AS(:,j),PS,mu,f]=feval(smoothHandle,smPars0,y,kalmanData,Zvars2,maxUn);
end;

lgd=[repmat('Random Walk, \lambda = ',size(nruLambda)) num2str(nruLambda)]; 
lgd2=[repmat('Productivity, \lambda = ',size(nruProdLambda)) num2str(nruProdLambda)];
if length(nruProdLambda)==1; k=1; else k=2; end;
figure; h=myPlot(time,[AS(:,k) ASrw graphUn]); legend(strvcat('Prod NAIRU',lgd,'Unemployment')); ttt=title('Unemployment and the NAIRU');
set(h(1),'linestyle','-'); set(h(3),'linestyle','--');  axis([-inf inf 0 11]); set(h(end),'color',cc.grey); set(ttt,'fontsize',7);

figure; h=myPlot(time,[AS graphUn]); legend(strvcat(lgd2,'Unemployment')); ttt=title('Unemployment and the NAIRU');
axis([-inf inf 0 11]); set(h(end),'color',cc.grey,'linestyle','-'); set(h(3),'linestyle','-.'); set(ttt,'fontsize',7);
labs=strvcat('Productivity',lgd); [specNru,omega] = spectrum([AS(:,k) ASrw],round(length(AS)/4),1,labs); 
[specNruP,omega] = spectrum(AS,round(length(AS)/4),1,lgd2);  

fCompProd=figure; subplot(211); h=myPlot(time,[AS(:,3) ASrw graphUn]); set(h,'linewidth',1);
lll=legend(strvcat('Productivity',lgd,'Unemployment')); ttt=title('Unemployment and the NAIRU - Random Walk Model');
axis([-inf inf 0 11]); set(h(end),'color',cc.grey,'linestyle','-'); set(h(3),'linestyle','-.'); set(h(4),'linestyle','-');
set(ttt,'fontsize',7); set(lll,'fontsize',6); set(gca,'fontsize',6);

subplot(212); h=myPlot(time,[AS(:,2:end) graphUn]); set(h,'linewidth',1);
lll=legend(strvcat(lgd2(2:end,:),'Unemployment')); ttt=title('Unemployment and the NAIRU - Productivity Model');
axis([-inf inf 0 11]); set(h(end),'color',cc.grey,'linestyle','-'); set(h(3),'linestyle','-'); 
set(ttt,'fontsize',7); set(lll,'fontsize',6); set(gca,'fontsize',6);

disp('Vars of RW    Productivity:'); varAS=repmat(nan,max([size(ASrw,2),size(AS,2)]),2); 
varAS(1:size(ASrw,2),1)=var(ASrw)'; varAS(1:size(AS,2),2)=var(AS)'; disp(varAS);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
t=toc; mints=floor(toc/60); secs=toc-60*mints;
fprintf('Time: %4.0f minutes, %4.2f seconds\n',mints,secs); diary off;
