% dataFile(:,1:2) ... dates
dataFile=aggr(dataFile,12,frequency,2); % aggregate the data
punew=dataFile(:,3); 
cpixfe=dataFile(:,4); % CPI X food and energy data not avalable till 1957
cpi_food=dataFile(:,5); 
cpi_oil=dataFile(:,6); 
gdpDefl=dataFile(:,7);  % GDP deflator quarterly only
un_16=dataFile(:,8); % unemployment 16+ both sexes
un_mm=dataFile(:,9); % unemployment married men, available from 1955:01 (OK)
un_25m=dataFile(:,10); % unemployment men 25-54
prodData=frequency*100*diff(log(dataFile(:,11:14))); 
pop=(100*diff(log(dataFile(:,15)))); pop=pop(pop~=0); pop=kron(pop,ones(frequency,1));
capU=dataFile(:,16); bp=dataFile(:,17);
prodData(:,1:2)=prodData(:,1:2)-repmat([repmat(mean(pop),frequency-1,1);pop],1,2);
time0=(dataFile(1):1/frequency:dataFile(end,1)+1-.001)';

inflCpi=frequency*100*log(punew(2:end)./punew(1:end-1)); 
shocks_ssw=frequency*100* ( .66*log( cpi_food(2:end)./cpi_food(1:end-1) ) ...
    + .34*log( cpi_oil(2:end)./cpi_oil(1:end-1) )  ) - inflCpi;
shocks_ssw=shocks_ssw-mean(shocks_ssw(~isnan(shocks_ssw))); names{3}='SShx'; names{4}='Nixon';

switch inflIndex
   case 1; infl=inflCpi; inflA=100*log(punew(frequency+1:end)./punew(1:end-frequency)); infLabel='DRI Inflation'; names{2}='dinDRI';
   case 2; infl=frequency*100*log(cpixfe(2:end)./cpixfe(1:end-1)); inflA=100*log(cpixfe(frequency+1:end)./cpixfe(1:end-frequency)); 
      infLabel='CPI ex Food and Energy Inflation'; names{2}='dinX';
end;      
switch unIndex
   case 1; un=un_16; unLabel='Unemployment 16+ both sexes'; names{1}='Un16+';
   case 2; un=un_mm; unLabel='Unemployment married men'; names{1}='UnMM';
   otherwise; un=un_25m; unLabel='Unemployment men 25-54';names{1}='Un25-54';
end;
fprintf('Inflation Series:     %s\n',infLabel); fprintf('Unemployment Series:  %s\n',unLabel);

un=un(persFromBeg:pers,:); infl=infl(persFromBeg-1:pers-1,:); sShocks=shocks_ssw(persFromBeg-1:pers-1,:); inflA=inflA(persFromBeg-1:end,:);
time=time0(persFromBeg+1+maxLags:pers); prodData2=prodData(persFromBeg:pers-1,:);  prodData=prodData(persFromBeg+maxLags:pers-1,:);
ip=prodData(:,1); gdp=prodData(:,2); nfProd=prodData(:,3); mProd=prodData(:,4);
deltaInfl=diff(infl); T=length(deltaInfl); y=deltaInfl(maxLags+1:end); dinflA=diff(inflA);
[foreInfl, actInfl] = arFore(infl,frequency,1);
switch prodIndex; case 1; prodSer=prodData2(:,3); case 2; prodSer=prodData2(:,4); end;

if shoxIndex==1; x=[un(2:end),deltaInfl,sShocks(2:end)]; 
   allLags=[allLags,shoxLags]; disp('Supply shocks included');
else x=[un(2:end),deltaInfl]; disp('Supply shocks NOT included');
   allLags=[unLags(1),inflLags(1); unLags(2),inflLags(2)]; end;

lagRes=mklagsMine(x,allLags); splineRegs=makeSplines(knots,splineOrder,length(y));
Xregs=lagRes.xl;
if shoxIndex; shRegs=Xregs(:,(maxInfl+maxUn)+1:end); else; shRegs=[]; end;
unXregs=Xregs(:,1:maxUn);
inflXregs=Xregs(:,maxUn+1:maxUn+maxInfl);
brRegs=[unXregs inflXregs shRegs];
X=[splineRegs unXregs inflXregs shRegs]; % design matrix
graphUn=X(:,splineSize+1);

% productivity
options=[];
[vE,fval,exitflag,output] = fminsearch(@urwLik,[12,1],options,prodSer); 
fprintf('Variances for Productivity:  %4.4f    %4.4f\n',vE(1)^2,vE(2)^2);
vE2=vE(1).^2*[1,prodLambda]; % 7/length(prodSer), prodLambda=.001
[AFp,PFp,ASp,PSp] = urwState(vE2,prodSer); 
[AFp2,PFp2,ASp2,PSp2] = urwState(vE.^2,prodSer);

switch prodDiffIndex;
    case 1; prodDiff=AFp-ASp; pDiffText='difference'; 
    case 2; prodDiff=AFp./ASp; pDiffText='ratio'; 
    case 3; prodDiff=ASp; pDiffText='prod trend';
end;


xP=mklagsMine([prodDiff x], [prodLags allLags]); Xprod=[splineRegs xP.xl]; Xprod2=xP.xl; 
prodM=mklagsMine(prodDiff, prodLags); prodMat=prodM.xl;
figure; myPlot(time,[ASp(maxLags+1:end) AFp(maxLags+1:end)]); axis([-inf inf 0 4]);
title(['US Productivity Growth, \lambda=' num2str(prodLambda)]); legend('Smoother','Filter');
figure; myPlot(time,[ASp(maxLags+1:end) ASp2(maxLags+1:end)]); title(['US Productivity Growth, \lambda=' num2str(prodLambda)]); legend('Imposed','Estimated');

% bandpass figure
graphProd=prodSer(length(prodSer)-length(graphUn)+1:end); graphInfl=infl(length(infl)-length(graphUn)+1:end); 
graphInflA=inflA(length(inflA)-length(graphUn)+1:end); graphDinflA=dinflA(length(dinflA)-length(graphUn)+1:end);
upF=15*frequency; trms=20*frequency; arTrms=frequency;
[bpProd, junk] = bpassar(graphProd,2,upF,trms,arTrms,0,0,frequency,1,6.25e-8);
[bpUn, junk] = bpassar(graphUn,2,upF,trms,arTrms,0,0,frequency,1,6.25e-8);
[bpInfl,junk] = bpassar(graphInfl,1,upF,trms,arTrms,0,0,frequency,1,6.25e-8);
[bpDinfl,junk] = bpassar(diff([graphInfl(1); graphInfl]),1,20*frequency,trms,frequency,0,0,frequency,1,6.25e-8);
prodTrend=graphProd-bpProd; unTrend=graphUn-bpUn; inflTrend=graphInfl-bpInfl; dinflTrend=diff([graphInfl(1); graphInfl])-bpDinfl;
figure; h=myPlot(time,[unTrend,prodTrend,graphUn]); title('Productivity and the NAIRU'); legend('NAIRU','Productivity','Unemployment',1);
set(h(2),'linestyle','-'); set(h(3),'linestyle','--'); axis([-inf inf 0 11]);
crr=corrcoef([prodTrend unTrend]);
fprintf('Corelation between the NAIRU and Productivity Trend:  %1.3f\n',crr(2));
if keyIndex; keyboard; end;