
% dataFile(:,1:2) ... dates
dataFile=aggr(dataFile,12,frequency,2); % aggregate the data
punew=dataFile(:,3); 
cpixfe=dataFile(:,4); % CPI X food and energy data not avalable till 1957
cpi_food=dataFile(:,5); 
cpi_oil=dataFile(:,6); 
gdpDefl=dataFile(:,7);  % GDP deflator quarterly only
un_16=dataFile(:,8); % unemployment 16+ both sexes
un_mm=dataFile(:,9); % unemployment married men, available from 1955:01 (OK)
un_25m=dataFile(:,10); % unemployment men 25-54
prodData=frequency*100*diff(log(dataFile(:,11:14))); 
pop=(100*diff(log(dataFile(:,15)))); pop=pop(pop~=0); pop=kron(pop,ones(frequency,1));
capU=dataFile(:,16); 
bp=dataFile(:,17);
prodData(:,1:2)=prodData(:,1:2)-repmat([repmat(mean(pop),frequency-1,1);pop],1,2);
time0=(dataFile(1):1/frequency:dataFile(end,1)+1-.001)';

% Transformations
inflCpi=frequency*100*log(punew(2:end)./punew(1:end-1)); 
shocks_ssw=frequency*100* ( .66*log( cpi_food(2:end)./cpi_food(1:end-1) ) ...
    + .34*log( cpi_oil(2:end)./cpi_oil(1:end-1) )  ) - inflCpi;
shocks_ssw=shocks_ssw-mean(shocks_ssw(~isnan(shocks_ssw))); names{3}='SShx'; names{4}='Nixon';

switch inflIndex
   case 1; infl=inflCpi; inflA=100*log(punew(frequency+1:end)./punew(1:end-frequency)); infLabel='DRI Inflation'; names{2}='dinDRI';
   case 2; infl=frequency*100*log(cpixfe(2:end)./cpixfe(1:end-1)); inflA=100*log(cpixfe(frequency+1:end)./cpixfe(1:end-frequency)); 
      infLabel='CPI ex Food and Energy Inflation'; names{2}='dinX';
   case 3; infl=frequency*100*log(gdpDefl(2:end)./gdpDefl(1:end-1)); inflA=100*log(gdpDefl(frequency+1:end)./gdpDefl(1:end-frequency)); 
      infLabel='GDP Deflator Inflation'; names{2}='dinGDP';
end;      
switch unIndex
   case 1; un=un_16; unLabel='Unemployment 16+ both sexes'; names{1}='Un16+';
   case 2; un=un_mm; unLabel='Unemployment married men'; names{1}='UnMM';
   otherwise; un=un_25m; unLabel='Unemployment men 25-54';names{1}='Un25-54';
end;
fprintf('Inflation Series:     %s\n',infLabel); fprintf('Unemployment Series:  %s\n',unLabel);
infl=infl(persFromBeg-1:pers-1,:);
if inflExpIndex==3; expDiffIndex=1; else expDiffIndex=0; end;
[foreInfl, actInfl] = arFore(infl,frequency,0,expDiffIndex); % AR inflation forecasts
switch inflExpIndex;
    case 1; expInfl=infl(1:end-1); labb='RW Inflation Expectations';
    case 2; expInfl=foreInfl(2:end); labb='AR Inflation Expectations -- Levels';
    case 3; expInfl=foreInfl(2:end); labb='AR Inflation Expectations -- Diffs';
end;
prodData2=prodData(persFromBeg:pers-1,:);  prodData=prodData(persFromBeg+maxLags:pers-1,:);
switch prodIndex; % 1 nonFarm, 2 manufacturing
    case 1; prodSer=prodData2(:,3); labb=[labb '    NonFarm Productivity'];
    case 2; prodSer=prodData2(:,4); labb=[labb '    Man Productivity'];
end; 

un=un(persFromBeg:pers,:); sShocks=shocks_ssw(persFromBeg-1:pers-1,:); inflA=inflA(persFromBeg-1:end,:);
time=time0(persFromBeg+1+maxLags:pers); 
ip=prodData(:,1); gdp=prodData(:,2); nfProd=prodData(:,3); mProd=prodData(:,4);
deltaInfl=infl(2:end)-expInfl; T=length(deltaInfl); y=deltaInfl(maxLags+1:end); dinflA=diff(inflA);

% productivity

[vE,fval,exitflag,output] = fminsearch(@urwLik,[12,1],[],prodSer); 
fprintf('Variances for Productivity:  %4.4f    %4.4f\n',vE(1)^2,vE(2)^2);
vE2=vE(1).^2*[1,prodLambda]; % 7/length(prodSer), prodLambda=.001
[AFp,PFp,ASp,PSp] = urwState(vE2,prodSer); 


upF=15*frequency; trms=20*frequency; arTrms=frequency;
[dprodbp, junk] = bpassar(diff(ASp),2,upF,trms,arTrms,0,0,frequency,1,6.25e-8);
dprodbp=diff(ASp)-dprodbp;
[bpProd1, junk] = bpassar(prodSer,2,upF,trms,arTrms,0,0,frequency,1,6.25e-8);
prodTrend1=prodSer-bpProd1; 
[prodTrend2, junk] = bpassar(prodSer,2,upF,trms,arTrms,0,0,frequency,3,6.25e-8);
prodDiff=AFp-ASp; pDiffText='difference'; 

if shoxIndex==1; x=[un(2:end),deltaInfl,sShocks(2:end)]; 
   allLags=[allLags,shoxLags]; disp('Supply shocks included');
else x=[un(2:end),deltaInfl]; disp('Supply shocks NOT included');
   allLags=[unLags(1),inflLags(1); unLags(2),inflLags(2)]; end;
switch prodIndKF;
    case 1; prodDiff=diff(ASp); prodDiff=[prodDiff(1); prodDiff]; disp([labb '   Prod Variable: Diff Prod Trend']); 
    case 2;
        dPser=diff(prodSer);
        [hpDprod, junk] = bpassar(dPser,2,2*upF,trms,arTrms,0,0,frequency,3,6.25e-8);
        pd1=diff(hpDprod); 
        prodDiff=[[repmat(pd1(1),2,1); pd1]];
        disp([labb '   Prod Variable: DiffDiff']);
    case 3;
        dPser=diff(prodSer);
        [hpDprod, junk] = bpassar(dPser,2,2*upF,trms,arTrms,0,0,frequency,3,6.25e-8);
        pd1=diff(hpDprod); 
        prodDiff=[[repmat(pd1(1),2,1); pd1]];
        pd2=diff(ASp); 
        prodDiff=[prodDiff [pd2(1); pd2]];
        disp([labb '   Prod Variable: Level and Diff']);
    case 4; 
        prodDiff=ASp;
        disp([labb '   Prod Variable: Level']);
    case 5; 
        prodDiff=diff(prodTrend1); prodDiff=[prodDiff(1); prodDiff];
        disp([labb '   Prod Variable: Diff Bandpass']);
end;
% Create Regression Matrices Now
%==========================================================================

lagRes=mklagsMine(x,allLags); splineRegs=makeSplines(knots,splineOrder,length(y));
Xregs=lagRes.xl;
if shoxIndex; shRegs=Xregs(:,(maxInfl+maxUn)+1:end); else; shRegs=[]; end;
unXregs=Xregs(:,1:maxUn); inflXregs=Xregs(:,maxUn+1:maxUn+maxInfl); brRegs=[unXregs inflXregs shRegs];
X=[splineRegs unXregs inflXregs shRegs]; % design matrix

xP=mklagsMine([prodDiff x], [prodLags allLags]); Xprod=[splineRegs xP.xl]; Xprod2=xP.xl; 
prodM=mklagsMine(prodDiff, prodLags); prodMat=prodM.xl;

% Bandpass Figure
graphUn=X(:,splineSize+1);
graphProd=prodSer(length(prodSer)-length(graphUn)+1:end); graphInfl=infl(length(infl)-length(graphUn)+1:end); 
graphInflA=inflA(length(inflA)-length(graphUn)+1:end); %graphDinflA=dinflA(length(dinflA)-length(graphUn)+1:end);
upF=15*frequency; trms=20*frequency; arTrms=frequency;
[bpProd, junk] = bpassar(graphProd,2,upF,trms,arTrms,0,0,frequency,1,6.25e-8);
[bpUn, junk] = bpassar(graphUn,2,upF,trms,arTrms,0,0,frequency,1,6.25e-8);
[bpInfl,junk] = bpassar(graphInfl,1,upF,trms,arTrms,0,0,frequency,1,6.25e-8);
[bpDinfl,junk] = bpassar(diff([graphInfl(1); graphInfl]),1,20*frequency,trms,frequency,0,0,frequency,1,6.25e-8); 
prodTrend=graphProd-bpProd; unTrend=graphUn-bpUn; inflTrend=graphInfl-bpInfl; dinflTrend=diff([graphInfl(1); graphInfl])-bpDinfl;
graphDprod=diff(prodTrend); graphDprod=[graphDprod(1); graphDprod];
bandpass=figure; h=myPlot(time,[unTrend,prodTrend,graphUn]); title('Productivity and the NAIRU'); 
legend('NAIRU','Productivity','Unemployment',1); axis([-inf inf -1 11]);
set(h(2),'linestyle','-','color',cc.liteGB); set(h(3),'linestyle','--','color',cc.darkBlue);
% figure; h=myplot(time,[prodTrend ma(graphProd,4)]); set(h,'linestyle','-'); set(h(2),'color',cc.liteGB);
% title('Productivity and Trend'); legend('Trend','Productivity Growth');
figure; h=myPlot(time,[ASp(maxLags+1:end) prodTrend ma(prodSer(maxLags+1:end),4)]); axis([-inf inf 0 4]);
title(['US Productivity Growth']); legend('Kalman Smoother','Bandpass','Actual',4); set(h(3),'linestyle','-');
axis([-inf inf -4 6]);
%set(h(2),'color',cc.liteGB,'linestyle','-');
% figure; myPlot(time,[ASp(maxLags+1:end) ASp2(maxLags+1:end)]); title(['US Productivity Growth, \lambda=' num2str(prodLambda)]); legend('Imposed','Estimated');

if keyIndex; keyboard; end;
yBp=unTrend; xBp=[ones(size(unTrend)) prodTrend]; bpHat=xBp\yBp; bpE=yBp-xBp*bpHat; bpVcv=var(bpE)*inv(xBp'*xBp);
rsq(1,1)=1-var(bpE)/var(yBp); rsq(2,1)=1-var(bpE)/var(yBp)*(size(xBp,1)-1)/(length(xBp)-2);
[ftstat,ftname]=regprint(bpHat,[],[],bpVcv,bpE,1,length(yBp),rsq,constIndex);

% Descriptive Statistics
year1=1973; if yearEnd>1995; year2=1995; else; year2=yearEnd-1; end;
descData=[prodTrend graphDprod graphUn graphInfl];
descData1=descData(time<year1,:); descData2=descData(year1<time<year2,:); descData3=descData(year2<time,:);
m1=mean(descData1)'; m2=mean(descData2)'; m3=mean(descData3)';
sd1=std(descData1)'; sd2=std(descData2)'; sd3=std(descData3)'; sds=[sd1 sd2 sd3]; ms=[m1 m2 m3];
descTab=[ms(1,:); sds(1,:); ms(2,:); sds(2,:); ms(3,:); sds(3,:); ms(4,:); sds(4,:)];
descTabSmall=[m1 m2 m3];

% productivity correlations
maxDiff=10;
crrUS=repmat(nan,maxDiff+1,2);
annDat=aggr([unTrend,prodSer(end-length(unTrend)+1:end)],frequency,1,2);
for jj=1:maxDiff;
        crrUS(jj,1)=kRegFiltUS(unTrend,prodSer(end-length(unTrend)+1:end),jj);
        crrUS(jj,2)=kRegFiltUS(annDat(:,1),annDat(:,2),jj);       
end;
crrTemp=corrcoef([unTrend,prodTrend]); crrUS(end,1)=crrTemp(2);
[annPtr, junk] = bpassar(annDat(:,2),2,15,10,2,0,0,1,3,6.25e-8);
crrTemp=corrcoef([annDat(:,1),annPtr]); crrUS(end,2)=crrTemp(2);

standData=[prodTrend unTrend dprodbp(maxLags:end)];
standData=(standData-repmat(mean(standData),length(standData),1))./repmat(std(standData),length(standData),1);
figure; h=myplot(time,standData); legend('Productivity','Unemployment','Diff Productivity',4); title('Standardized Trends');
corrstand=corrcoef(standData); corrstand1=corrstand(2,1); corrstand2=corrstand(2,3); corrstand1=myRound(corrstand1,2); corrstand2=myRound(corrstand2,2);
xlab=['Corr(Un, Prod) = ' num2str(corrstand1) ',   Corr(Un, Diff Prod) = ' num2str(corrstand2)]; xlabel(xlab);
set(h(3),'linestyle','-');
